@@ -8,7 +8,7 @@ import ctypes
 import threading
 import asyncio
 import math
-import pylimo.limomsg
+import pylimo.limomsg as limomsg
 
 import serial.tools.list_ports
 
@@ -324,105 +324,131 @@ class LIMO:
         self.left_angle_scale = 2.47
         self.right_angle_scale = 1.64
         self.max_inner_angle = 0.48869
-    def GetOdomMetry(self):
-        wz=0.0
-        vx=0.0
-        vy=0.0
-        if(limomsg.GetMotionMode()==0x00):
-            vx=limomsg.GetLinearVelocity()
-            vy=0.0
-            wz=limomsg.GetAngularVelocity()
-            return (vx,vy,wz)
-        elif(limomsg.GetMotionMode()==0x01):            
-            linear_velocity=limomsg.GetLinearVelocity()
-            inner_angle = limomsg.GetSteeringAngle()
-            #r = self.wheelbase/ math.tan(math.fabs(inner_angle)) + self.track / 2.0
-            central_angle  = self.ConvertInnerAngleToCentral(inner_angle)
-            if (central_angle > 0):
-                try:
-                    wz = linear_velocity / (self.wheelbase/ math.tan(math.fabs(inner_angle)) + self.track / 2.0)
-                except ZeroDivisionError:
-                    wz = linear_velocity/(self.track / 2.0)
-            else :
-                try:
-                    wz = -linear_velocity / (self.wheelbase/ math.tan(math.fabs(inner_angle)) + self.track / 2.0)
-                except ZeroDivisionError:
-                    wz = -linear_velocity /(self.track / 2.0)  
-            vx = linear_velocity * math.cos(central_angle)
-            if (linear_velocity >= 0.0) :
-                vy = linear_velocity * math.sin(central_angle)
+        self.max_lateral_velocity=1.0
+    # def GetOdomMetry(self):
+    #     wz=0.0
+    #     vx=0.0
+    #     vy=0.0
+    #     if(limomsg.GetMotionMode()==0x00):
+    #         vx=limomsg.GetLinearVelocity()
+    #         vy=0.0
+    #         wz=limomsg.GetAngularVelocity()
+    #         return (vx,vy,wz)
+    #     elif(limomsg.GetMotionMode()==0x01):            
+    #         linear_velocity=limomsg.GetLinearVelocity()
+    #         inner_angle = limomsg.GetSteeringAngle()
+    #         #r = self.wheelbase/ math.tan(math.fabs(inner_angle)) + self.track / 2.0
+    #         central_angle  = self.ConvertInnerAngleToCentral(inner_angle)
+    #         if (central_angle > 0):
+    #             try:
+    #                 wz = linear_velocity / (self.wheelbase/ math.tan(math.fabs(inner_angle)) + self.track / 2.0)
+    #             except ZeroDivisionError:
+    #                 wz = linear_velocity/(self.track / 2.0)
+    #         else :
+    #             try:
+    #                 wz = -linear_velocity / (self.wheelbase/ math.tan(math.fabs(inner_angle)) + self.track / 2.0)
+    #             except ZeroDivisionError:
+    #                 wz = -linear_velocity /(self.track / 2.0)  
+    #         vx = linear_velocity * math.cos(central_angle)
+    #         if (linear_velocity >= 0.0) :
+    #             vy = linear_velocity * math.sin(central_angle)
             
-            else :
-                vy = linear_velocity * math.sin(-central_angle)
+    #         else :
+    #             vy = linear_velocity * math.sin(-central_angle)
             
-            return (vx,vy,wz)
-        elif(limomsg.GetMotionMode() ==0x02):
-            vx = limomsg.GetLinearVelocity()
-            vy = limomsg.GetLateralVelocity
-            wz = limomsg.GetAngularVelocity()
-            return (vx,vy,wz)
-        else:return (vx,vy,wz)
-    def SetMotionCommand(self,
+    #         return (vx,vy,wz)
+        # elif(limomsg.GetMotionMode() ==0x02):
+        #     vx = limomsg.GetLinearVelocity()
+        #     vy = limomsg.GetLateralVelocity
+        #     wz = limomsg.GetAngularVelocity()
+        #     return (vx,vy,wz)
+        # else:return (vx,vy,wz)
+    # def SetMotionCommand(self,
                          
-                         linear_x: float = 0.0,
-                         linear_y: float = 0.0,
-                         angular_z: float = 0.0):                
-        '''
-        fout diff  input linear_x and angular_z  
-        ackermann  input linear_x and angular_z
-        mcnamu     input linear_x linear_y angular_z
-        '''
-        if(limomsg.GetMotionMode()==0x00):
-            self.device.SetMotionCommand(
-            linear_x, angular_z, 0, 0)
-        elif(limomsg.GetMotionMode()==0x01):
-            try:
-           # r=ctypes.c_double(linear_x/angular_z)
-                central_angle = math.atan(self.wheelbase*angular_z/linear_x)
-            except ZeroDivisionError:
-                central_angle=0
-            inner_angle = self.ConvertCentralAngleToInner(central_angle)
-            if (inner_angle > self.max_inner_angle):
-                inner_angle = self.max_inner_angle
+    #                      linear_x: float = 0.0,
+    #                      linear_y: float = 0.0,
+    #                      angular_z: float = 0.0):                
+    #     '''
+    #     fout diff  input linear_x and angular_z  
+    #     ackermann  input linear_x and angular_z
+    #     mcnamu     input linear_x linear_y angular_z
+    #     '''
+    #     if(limomsg.GetMotionMode()==0x00):
+    #         self.device.SetMotionCommand(
+    #         linear_x, angular_z, 0, 0)
+    #     elif(limomsg.GetMotionMode()==0x01):
+    #         try:
+    #        # r=ctypes.c_double(linear_x/angular_z)
+    #             central_angle = math.atan(self.wheelbase*angular_z/linear_x)
+    #         except ZeroDivisionError:
+    #             central_angle=0
+    #         inner_angle = self.ConvertCentralAngleToInner(central_angle)
+    #         if (inner_angle > self.max_inner_angle):
+    #             inner_angle = self.max_inner_angle
             
-            if (inner_angle < -self.max_inner_angle):
-                inner_angle = -self.max_inner_angle
-            if (inner_angle > 0) :
-                steering_angle = inner_angle / self.left_angle_scale
-            else :
-                steering_angle = inner_angle / self.right_angle_scale
-            self.device.SetMotionCommand(
-            linear_x, 0, 0, steering_angle)
-            # self.SetMotionCommand(linear_x, 0, 0, steering_angle)
-        elif(limomsg.GetMotionMode() ==0x02):
-            self.device.SetMotionCommand(
-            linear_x, angular_z, linear_y,0 )
-        else:print("motion mode not supported in receive cmd_vel")
+    #         if (inner_angle < -self.max_inner_angle):
+    #             inner_angle = -self.max_inner_angle
+    #         if (inner_angle > 0) :
+    #             steering_angle = inner_angle / self.left_angle_scale
+    #         else :
+    #             steering_angle = inner_angle / self.right_angle_scale
+    #         self.device.SetMotionCommand(
+    #         linear_x, 0, 0, steering_angle)
+    #         # self.SetMotionCommand(linear_x, 0, 0, steering_angle)
+    #     elif(limomsg.GetMotionMode() ==0x02):
+    #         self.device.SetMotionCommand(
+    #         linear_x, angular_z, linear_y,0 )
+    #     else:print("motion mode not supported in receive cmd_vel")
             
-        # self.device.SetMotionCommand(
-        #     linear_vel, angular_vel, lateral_velocity, steering_angle)
-    def ConvertInnerAngleToCentral(self,inner_angle):
-        try:
+    #     # self.device.SetMotionCommand(
+    #     #     linear_vel, angular_vel, lateral_velocity, steering_angle)
+    # def ConvertInnerAngleToCentral(self,inner_angle):
+    #     try:
 
-            r = self.wheelbase / math.tan(math.fabs(inner_angle)) + self.track / 2
-        except ZeroDivisionError:
-                r=self.track / 2
-        central_angle = math.atan(self.wheelbase / r)
+    #         r = self.wheelbase / math.tan(math.fabs(inner_angle)) + self.track / 2
+    #     except ZeroDivisionError:
+    #             r=self.track / 2
+    #     central_angle = math.atan(self.wheelbase / r)
 
-        if (inner_angle < 0): 
-            central_angle = -central_angle
+    #     if (inner_angle < 0): 
+    #         central_angle = -central_angle
         
 
-        return central_angle
+    #     return central_angle
 
-    def ConvertCentralAngleToInner(self,central_angle):
-        inner_angle = math.atan(2 * self.wheelbase* math.sin(math.fabs(central_angle)) /
-                                   (2 * self.wheelbase * math.cos(math.fabs(central_angle)) -
-                                    self.track * math.sin(math.fabs(central_angle))))
+    # def ConvertCentralAngleToInner(self,central_angle):
+    #     inner_angle = math.atan(2 * self.wheelbase* math.sin(math.fabs(central_angle)) /
+    #                                (2 * self.wheelbase * math.cos(math.fabs(central_angle)) -
+    #                                 self.track * math.sin(math.fabs(central_angle))))
 
-        if (central_angle < 0):
-            inner_angle = -inner_angle
-        return inner_angle
+    #     if (central_angle < 0):
+    #         inner_angle = -inner_angle
+    #     return inner_angle
+
+    def SetMotionCommand(self,
+                         linear_vel: float = 0.0,
+                         angular_vel: float = 0.0,
+                         lateral_velocity: float = 0.0,
+                         steering_angle: float = 0.0):
+        if (steering_angle > self.max_inner_angle):
+                steering_angle = self.max_inner_angle
+            
+        if (steering_angle < -self.max_inner_angle):
+            steering_angle = -self.max_inner_angle
+
+        if (angular_vel > self.left_angle_scale):
+            angular_vel = self.left_angle_scale
+            
+        if (angular_vel < -self.left_angle_scale):
+            angular_vel = -self.left_angle_scale 
+
+        if (lateral_velocity > self.max_lateral_velocity):
+            lateral_velocity = self.max_lateral_velocity
+            
+        if (lateral_velocity < -self.max_lateral_velocity):
+            lateral_velocity = -self.max_lateral_velocity 
+
+        self.device.SetMotionCommand(linear_vel,angular_vel,lateral_velocity,steering_angle)    
 
 
     def EnableCommand(self):
@@ -443,6 +469,18 @@ class LIMO:
         '''
         return limomsg.GetAngularVelocity()
 
+    def GetSteeringAngle(self):
+            '''
+            Get the steering angle from limo
+            '''
+            return limomsg.GetSteeringAngle()
+
+    def GetLateralVelocity(self):
+            '''
+            Get the lateral velocity from limo
+            '''
+            return limomsg.GetLateralVelocity()
+
     def GetControlMode(self):
         '''
         Get the control mode from limo
